import java.io.IOException;
import java.math.BigInteger;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.SysexMessage;
import javax.sound.midi.MidiDevice.Info;


public class SendSysEx {

	private final static byte PRODUCT_MODEL_ID_BYTE_INDEX = 3;
	private final static byte PRODUCT_MODEL_ID_APC_20 = 0x7b;
	private final static byte PRODUCT_MODEL_ID_APC_40 = 0x73;

	private final static byte MODE_BYTE_INDEX = 7;
	private final static byte MODE_GENERIC = 0x40;
	private final static byte MODE_ABLETON_LIVE = 0x41;
	private final static byte MODE_ABLETON_LIVE_ALTERNATE = 0x42;

	private final static String templateMessage = "f0 47 00 FF 60 00 04 FF 08 01 02 f7";


	private static byte productModelId = 0x00;
	private static byte mode = MODE_ABLETON_LIVE;
	
	private static int port = -1;



	public static void main(String[] args) throws IOException, MidiUnavailableException, InvalidMidiDataException {
		findAPC();
		send();
	}


	public static void findAPC(){
		int i = 0;
		Info[] infos = MidiSystem.getMidiDeviceInfo();
		for(Info info : infos){
			if(info.getClass().getName().contains("MidiOutDeviceInfo")){
				if(info.getName().contains("Akai APC")){
					System.out.println("Found " + info.getName() + " at port " + i  );
					port = i;
					if(info.getName().contains("40")){
						productModelId = PRODUCT_MODEL_ID_APC_40;
					}else{
						productModelId = PRODUCT_MODEL_ID_APC_20;
					}

					return; 
				}
			}
			i++;
		}

		System.out.println("No APC controller found. ");
		System.exit(-1);
	}


	public static void send() throws MidiUnavailableException, InvalidMidiDataException{
	
		Info[] infos = MidiSystem.getMidiDeviceInfo();
		Receiver receiver;
		MidiDevice device = null;
		
		try {
			device = MidiSystem.getMidiDevice(infos[port]);
			device.open();

			System.out.println("port open.");

			receiver = device.getReceiver();

			SysexMessage sysexMessage = createMessage();

			receiver.send(sysexMessage, -1);
			System.out.println("sent message: " + byteArrayToHexString(sysexMessage.getMessage()));

		}finally{
			if(null != device){
				device.close();
				System.out.println("done.");
			}
		}
	}

	private static byte[] hexStringToByteArray(String hex){
		hex = hex.replaceAll(" ", "");
		byte[] bts = new byte[hex.length() / 2];
		for (int i = 0; i < bts.length; i++) {
			bts[i] = (byte) Integer.parseInt(hex.substring(2*i, 2*i+2), 16);
		}
		
		return bts;
	}
	
	private static String byteArrayToHexString(byte[] bytes){
		BigInteger bi = new BigInteger(1, bytes);
		
		String hexString = bi.toString(16);
		
		StringBuffer buf = new StringBuffer();
		for(int i = 0; i < hexString.length(); i+=2){
			if(i+2 < hexString.length()){
				buf.append(hexString.substring(i, i+2));
				buf.append(" ");
			}else{
				buf.append(hexString.substring(i));
			}
		}
		
		return buf.toString();
	}
	
	private static SysexMessage createMessage() throws InvalidMidiDataException {
		byte[] message = hexStringToByteArray(templateMessage);
		
		message[PRODUCT_MODEL_ID_BYTE_INDEX] = productModelId;
		message[MODE_BYTE_INDEX] = mode;
		
		SysexMessage msg = new SysexMessage();
		msg.setMessage(message, message.length);
		return msg;
	}

	
	public static void printAvailablePorts(){
		System.out.println("Midi Ports:");
		int i = 0;

		Info[] infos = MidiSystem.getMidiDeviceInfo();
		for(Info info : infos){
			System.out.println(i + ": "+ info);
			i++;
		}
	}

}
